﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Documents.Reports.Report.Section;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Documents.Reports.Report.Text;
using Infragistics.Documents.Reports.Report;
using Infragistics.Documents.Reports.Report.Projection;
using Infragistics.Documents.Reports.Report.Grid;
using System.Diagnostics;
using Infragistics.Win.Printing;


namespace WinGridPrintColumnOnEachePage
{
    public partial class Form1 : Form
    {

        IProjectionPageCollection pages = null;
        DataTable tab = new DataTable();
        int currentPrintPageNumber = 0;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
            tab.Columns.Add("Fixed", typeof(string)).DefaultValue = "Fixed";
            for(int i = 1; i < 15; i++)
                tab.Columns.Add("Column"+i.ToString(), typeof(string)).DefaultValue = "Text";
            for (int i = 1; i < 150; i++)
                tab.Rows.Add();
            ultraGrid1.DataSource = tab;

            label1.DataBindings.Add(new Binding("Text", trackBar1, "Value"));
            
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            tab.Columns.Add("Column" + tab.Columns .Count .ToString(), typeof(string)).DefaultValue = "Text";
        }

        private Report GenerateReport(UltraGrid grid, UltraPrintDocument doc)
        {
            UltraGridColumn[] columns = grid.DisplayLayout.Bands[0].Columns.Cast<UltraGridColumn>().ToArray<UltraGridColumn>();

            HideColumns(columns, true);

            Report rep = new Report();

            float width = Infragistics.Documents.Reports.Utils.Converter.PointsToPixels(doc.DefaultPageSettings.PrintableArea.Width -
                doc.DefaultPageSettings.Margins.Left - doc.DefaultPageSettings.Margins.Right) * 1.7f;

            int curColumnIndex = 1;
            float curWidth = Infragistics.Documents.Reports.Utils.Converter.PixelsToPoints(columns[0].Width);
            while (curColumnIndex < ultraGrid1.DisplayLayout.Bands[0].Columns.Count)
            {
                float curColWidth = Infragistics.Documents.Reports.Utils.Converter.PixelsToPoints(columns[curColumnIndex].Width);
                if (curWidth + curColWidth < width)
                {
                    columns[curColumnIndex].Hidden = false;
                    curWidth += curColWidth;
                    curColumnIndex++;
                }
                else
                {
                    Export(ultraGrid1, rep);
                    curWidth = Infragistics.Documents.Reports.Utils.Converter.PixelsToPoints(columns[0].Width);
                    HideColumns(columns, true);
                }
            }

            Export(grid, rep);
            HideColumns(columns, false);
            pages = rep.Generate();
            return rep;
        }

        private void ultraButton2_Click(object sender, EventArgs e)
        {

            GenerateReport(ultraGrid1, ultraPrintDocument1);
            currentPrintPageNumber = 0;
            ultraPrintPreviewControl1.InvalidatePreview();
           
        }

        private void HideColumns(UltraGridColumn[] columns, bool hide)
        {
            
            if (hide)
            {
                List<UltraGridColumn> visibleColumn = columns.Where(c => (c.Key != "Fixed" && !c.Hidden)).ToList<UltraGridColumn>();
                visibleColumn.ForEach(c => c.Hidden = !c.Hidden);
            }
            else
            {
                List<UltraGridColumn> visibleColumn = columns.ToList<UltraGridColumn>();
                visibleColumn.ForEach(c => c.Hidden = false);
            }

        }

        private void Export(UltraGrid ultraGrid1, Report rep)
        {
            ISection section = rep.AddSection();
            this.ultraGridDocumentExporter1.Export(ultraGrid1, section);
        }

        private void ultraPrintDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            if (pages == null)
                return;
            Infragistics.Documents.Reports.Graphics.Graphics reportGraphics = new Infragistics.Documents.Reports.Graphics.Graphics(e.Graphics);

            // Save the state of the Graphics object So we can restore it later. 
            reportGraphics.SaveState();

            // Get the page margins in points. 
            float leftMarginInPoints = (e.MarginBounds.Left / 100f) * 72f;
            float topMarginInPoints = (e.MarginBounds.Top / 100f) * 72f;

            // Get the available print width and height in points. This is the size
            // of the printable area of the page inside the margins. 
            float availablePrintWidth = (e.MarginBounds.Width / 100f) * 72f;
            float availablePrintHeight = (e.MarginBounds.Height / 100f) * 72f;

            // Use a translate to shift the report page drawing inside the printer page margins. 
            reportGraphics.Translate(leftMarginInPoints, topMarginInPoints);

            // Get the currently printing report page.
            IProjectionPage reportPage = this.pages[this.currentPrintPageNumber];

            // Scale the report page so that it fits inside the available print area. 
            float widthRatio = availablePrintWidth / reportPage.Width;
            float heightRatio = availablePrintHeight / reportPage.Height;
            reportGraphics.Scale(widthRatio, heightRatio);

            // Draw the page. 
            reportPage.Draw(reportGraphics);

            // Restore the Graphics settings. 
            reportGraphics.RestoreState();

            // Increment the page number. 
            currentPrintPageNumber += 1;

            // If this is the last page, set HasMorePages to false. 
            e.HasMorePages = currentPrintPageNumber < (pages.Count);

            if (currentPrintPageNumber >= (pages.Count))
            {
                currentPrintPageNumber = 0;
            }
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            ultraPrintPreviewControl1.Settings.Zoom = trackBar1.Value / 100;
        }

        private void ultraButton3_Click(object sender, EventArgs e)
        {
            ultraPrintDocument1.Print();
        }

        private void ultraButton4_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofd = new SaveFileDialog();
            ofd.Filter = "PDF | *.pdf";
            if (ofd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Report rep = GenerateReport(ultraGrid1, ultraPrintDocument1);
                rep.Publish(ofd.FileName, FileFormat.PDF);
                Process.Start(ofd.FileName);
            }
        }

        private void ultraButton5_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofd = new SaveFileDialog();
            ofd.Filter = "PDF | *.pdf";
            if (ofd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Report rep = GenerateReport(ultraGrid1, ultraPrintDocument1);
                rep.Publish(ofd.FileName, FileFormat.XPS);
                Process.Start(ofd.FileName);
            }
        }
    
    }
}
